==> snf_3.3.0_webrelease.tar.gz: snf_3.3.0_webrelease/src/snfdc/rslave.r <==
CC****************************************************************************** C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
      subroutine rslave(co,ilab,jstepa,nmode,freq,intonly,nummodes)
C>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
C re-written by J. Neugebauer, Uni Erlangen, 2003
CCH extended by C. Herrmann, Uni Bonn, 02/2005 (intonly tool),
CCH using code parts
CCH from ANF, version 1.1.0, written by J. Neugebauer and B.A. Hess, 2001
CCH modified by C. Herrmann, Uni Jena 04/05 ( ADF-interface)
CCH modified by C. Herrmann, Uni Jena 05/05 ( Gaussian interface)
C>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      include 'defs.h'
      include 'comm.h'
      include 'param.h'  
      INCLUDE 'objects.h'
      integer ibuf,info,nn,iat,idum,ixyz,irc,idir,l,mypid,ir1,
     * idmy,j,mbuf,diraus,ldal,iclean,icleanl,mcharge,lo
      real*8 escf,gsum
      character*255 dalname
      character*10 prtype,calctype

      integer lprop,lpol,ldscf,xprog,ldmol
      integer xprogb

      integer*4 itst,jstepa
      character*255 b1
      character*5 nic
      character*80 sdum,dalmw
      character*24 date,dmname
      character*255 dscfname,propname,polname
      real*8 cpu,wall,gcpu,gwall
      integer nit
      integer xeq,tmpxeq,icmp,idalmw,ldalmw
      real*8 co(3,natoms)
      real*8 dipal(3),dipbe(3)
      integer ilab(natoms)
      real*8 nmode(maxmode,maxatoms*3),freq(maxmode)
      integer intonly,nummodes,im,imode,ipre,i,disptype,k
      real*8 normmodes(natoms*3)
      logical valdtype

      logical found,snffast
C#    integer w,st(MPI_STATUS_SIZE)
C#    data w /MPI_COMM_WORLD/
 
      nic='nice '
      ir1=3
      lo=2
c receive step range     
C#    CALL MPI_RECV(itst,1,MPI_INTEGER,tid0,410,w,st,info)
C#    CALL mpierr('MPI_RECV itst','rslave',info)
C#    CALL MPI_RECV(jstepa,1,MPI_INTEGER,tid0,411,w,st,info)
C#    CALL mpierr('MPI_RECV jstepa','rslave',info)
C//SKIPC#
      CALL PVMFRECV(tid0,0,ibuf)
      CALL PVMFUNPACK(INTEGER4,itst  ,1,1,info)
      CALL PVMFUNPACK(INTEGER4,jstepa,1,1,info)
C//END

      write (iosl,*) '>>> slave calc on tid ',mytid
      write (iosl,*) 'TID0=',tid0,' STEP ',jstepa

C----------------------
C read snf_control file
C----------------------
      call rscontrol(tempdir(1:lt)//'/snf_control',progno,calcno,ipol,
     *  omeg,snffast,iclean,icleanl,naus,mxomeg,cstep,idbg)

C-----------------------------
C Turbomole: read control file
C-----------------------------
      if(progno.eq.1) then 
        call rcontrol(tempdir(1:lt)//'/control',rpast,smult,isuhf,
     *   natoms,idbg)

C----------------------
C Turbomole: read coord
C----------------------
        call rcoord(tempdir(1:lt)//'/coord.ori',nn,co,ilab)
        if(nn.ne.natoms) then
           write(iosl,*) 'natoms different on coord and control'
C//SKIPC#1
           call pvmfexit(info)
           stop
        end if

C-------------------------------
C DALTON: read molecule.mol file
C-------------------------------
      else if(progno.eq.2) then 
        open(ir1,file=tempdir(1:lt)//'/molecule.mol.ori')
        call rwdalmol(natoms,ilab,co,ir1,natoms,mcharge,'read',lo)
        close(ir1)
C-------------------------------------------
C Gaussian: read coords from gaussian.com file
C-------------------------------------------
      else if(progno.eq.3) then
        call rgausscoord(tempdir(1:lt)//'/gaussian.com',natoms,co,
     *  ilab)
C-------------------------------
C ADF: read coords from adf.in file
C-------------------------------
      else if(progno.eq.4) then 
        call radfcoord(tempdir(1:lt)//'/adf.in',natoms,co,ilab,iosl)
      else
*----------------------
* other program modes ?
*----------------------
        write(idbg,*) 'program mode currently not available'
        call flush(idbg)
        call errex('program mode not available')
      end if

*---------------------------------------
* determine atom, xyz and dir for step i
*---------------------------------------
      if(nummodes.eq.0) then
        iat=jstepa/(3*naus)
        if(mod(jstepa,(3*naus)).ne.0.or.iat.eq.0) iat=iat+1
        idum=(jstepa-(3*naus)*(iat-1))
        ixyz=idum/naus
        if(mod(idum,naus).ne.0) ixyz=ixyz+1
        diraus=mod(idum,naus)
        if(diraus.eq.0) diraus=naus
        idir=naus/2-diraus+1
          if(idir.le.0) idir=idir-1
  
        write(iosl,*) 'step       =        ',jstepa
        write(iosl,*) 'atom       =        ',iat
        write(iosl,*) 'coordinate =        ',ixyz
        write(iosl,*) 'idir       =        ',idir

*-----------------------------------------------------
* intensities only mode: 
* determine mode no., and prefactor for displacement
* along this mode
*-----------------------------------------------------
      else
* mode number:
        im=imode(jstepa,naus)

* prefactor for displacement:
        idir=ipre(jstepa,naus)

        write(iosl,*) 'step       =        ',jstepa
        write(iosl,*) 'mode no.   =        ',im
        write(iosl,*) 'prefactor  =        ',idir
  
        i=im
*---------------------------------------------------------
* normalize normal coordinates (in principle, there should
* be normalized normal coordinates in "snf_control")
*---------------------------------------------------------
CCH        call normalize(nmode,3*natoms)
      end if

      if(nummodes.eq.0) then
*---------------------------------------------
* change coordinate - displacement of one atom
*---------------------------------------------
        write(iosl,*) 'COORD = ',co
        call flush(iosl)
        co(ixyz,iat)=co(ixyz,iat)+idir*cstep
      else
*-----------------------------------------------------
* change coordinate - displacement along normal modes 
* (intensities only program mode)
*-----------------------------------------------------
        call rkeyint(tempdir(1:lt)//'/snf_control','$disptype',disptype)
        write(iosl,*) 'disptype: ',disptype
        if(.not.valdtype(disptype)) call errex('disptype not valid')
        if(disptype.eq.1) then
          call displacecoord(nmode,co,ilab,i,idir)
        else if(disptype.eq.2) then
          call memmgr(memgdw,'vibf    ',s8,pvib,3*natoms)
          call getfrtm(tempdir(1:lt)//'/control',natoms,s8(pvib))
          call displacecoord2(normmodes,natoms,
     *         co,ilab,i,cstep,idir,s8(pvib+i-1))
          call memmgr(memfdw,'vibf    ',s8,pvib,3*natoms)
        end if
      end if
*---------------------------------------------
* write new coordinate file 
*---------------------------------------------
        call prog_type(progno,prtype)
        call calc_type(progno,calcno,calctype)
        write(iosl,*) 'program to be used for single points: ',prtype
        write(iosl,*) 'type of calculation: ',calctype
        if(progno.eq.1) then
          call wcoord(tempdir(1:lt)//'/coord',natoms,co,ilab)
        else if(progno.eq.2) then
          open(ir1,file=tempdir(1:lt)//'/molecule.mol.ori')
          open(lo,file=tempdir(1:lt)//'/molecule.mol')
          call rwdalmol(natoms,ilab,co,ir1,natoms,mcharge,'writ',lo)
          close(lo)
          close(ir1)
        else if(progno.eq.3) then
          write(iosl,*) 'rslave: calling wgauss ... tempdir = '//
     *tempdir(1:lt)
          call flush(iosl)
          call wgauss(tempdir(1:lt),natoms,ilab,co)
        else if(progno.eq.4) then
          write(iosl,*) 'rslave: calling wadf ... tempdir = '//
     *tempdir(1:lt)
          call flush(iosl)
          call wadf(tempdir(1:lt),natoms,ilab,co,iosl) 
        end if

      return
      end 
************************************************************************
      INTEGER FUNCTION xprog(p,f)
      INCLUDE 'defs.h'
      INCLUDE 'param.h'
      CHARACTER*(*) p,f
      CHARACTER*255 cmdout
      INTEGER info,tmpxeq
      CHARACTER*24 date
      CALL fdate(date)
      CALL concat(p,' >',cmdout)
      CALL appba(cmdout,f)
C@    WRITE (iosl,*) date,cmdout
      info=tmpxeq(cmdout)
C@    WRITE (iosl,*) date,' RC=',info
      IF (info.NE.0) THEN
       WRITE (iosl,*) p,' failed; process shut down. mytid=',mytid
       CALL slshutdown(99,tid0,mytid)
      ENDIF
      xprog=info
      RETURN
      END
************************************************************************
      SUBROUTINE slshutdown(ityp,loctid0,loctid)
      IMPLICIT none
      INTEGER ityp,loctid0,loctid
      INCLUDE 'comm.h'
      INCLUDE 'param.h'
      INTEGER*4 tid4,ityp4
      INTEGER mbuf,info
C#    INTEGER w
C#    DATA w /MPI_COMM_WORLD/

      CALL termpid(0)
C:    CALL MPI_SSEND(mytid,1,MPI_INTEGER,tid0,fsys_mpitag,w,info)
C:    CALL mpierr('MPI_SSEND mytid','slshutdown',info)
C:    CALL MPI_SSEND(fsys_taskstop,1,MPI_INTEGER,tid0,501,w,info)
C:    CALL mpierr('MPI_SSEND mnotify','slshutdown',info)
C:    CALL MPI_SSEND(ityp,1,MPI_INTEGER,tid0,601,w,info)
C:    CALL mpierr('MPI_SSEND ityp','slshutdown',info)
C//SKIPC:
C#    CALL MPI_SEND(mytid,1,MPI_INTEGER,tid0,fsys_mpitag,w,info)
C#    CALL mpierr('MPI_SEND mytid','slshutdown',info)
C#    CALL MPI_SEND(fsys_taskstop,1,MPI_INTEGER,tid0,501,w,info)
C#    CALL mpierr('MPI_SEND mnotify','slshutdown',info)
C#    CALL MPI_SEND(ityp,1,MPI_INTEGER,tid0,601,w,info)
C#    CALL mpierr('MPI_SEND ityp','slshutdown',info)
C//END
C//SKIPC#
      IF (loctid0.NE.PVMNOPARENT .AND. loctid.GE.0) THEN
** notify parent that this slave stops
       CALL PVMFINITSEND(PVMDATADEFAULT,mbuf)
       tid4=loctid
       CALL PVMFPACK(INTEGER4,tid4,1,1,info)
       ityp4=ityp
       CALL PVMFPACK(INTEGER4,ityp4,1,1,info)
       CALL PVMFSEND(loctid0,BAHTASKSTOP,info)
       IF (info.NE.0) CALL PVMFPERROR('[slshutdown] send stop msg',info)
      ENDIF
C//END
      CALL termcomm('rslave')
** termcomm will not return
      STOP 'miracle in rslave'
      END

CMR170202
      INTEGER FUNCTION xprogb(p,f)
      INCLUDE 'defs.h'
      INCLUDE 'param.h'
      CHARACTER*(*) p,f
      CHARACTER*255 cmdout
      INTEGER info,tmpxeq
      CHARACTER*24 date
      CALL fdate(date)
      CALL concat(p,' >',cmdout)
      CALL appba(cmdout,f)
C@    WRITE (iosl,*) date,cmdout
      info=tmpxeq(cmdout)
C@    WRITE (iosl,*) date,' RC=',info
      IF (info.NE.0) THEN
        WRITE (iosl,*) 'ERROR: ',p,' failed '
      ENDIF
      xprogb=info
      RETURN
      END
CMR170202
      
      subroutine actest(progn,found)
      include 'defs.h'
      include 'param.h'
      character*(*) progn
C      character*7  actual
      integer info, tmpxeq,lprog
      logical found
C      parameter(actual='$actual')
 
      found=.false.
      call llen(progn,lprog)
C ... this caused problem on a SGI machine:
C      call findstr(tempdir(1:lt)//'/control',actual(1:7),found)
      call findstr(tempdir(1:lt)//'/control','$actual',found)
      if(found) then
        write(iosl,*) '** actest: found $actual in control'
        write(iosl,*) '**         program in trouble: ',progn(1:lprog)
        call flush(iosl)
        if(lb.eq.0) then
          info=tmpxeq('actual -r')
        else
          info=tmpxeq(progdir(1:lb)//'../scripts/actual -r')
        end if
        if(info.ne.0) then
          write(iosl,*) '** actest: could not execute actual -r'
          call flush(iosl)
          call slshutdown(99,tid0,mytid)
        end if
      end if
      end

==> snf-4.0.0_webrelease.tar.gz: snf-4.0.0_webrelease/src/snfdc/rslave.r <==
CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************

      subroutine rslave(co,ilab,jstepa,nmode,freq,intonly,nummodes,nic)

c     ================================================================
c     purpose: prepare input files for electronic structure programs 
c              for displaced coordinates
c   
c     history:
c       - original version by B.A. Hess was Turbomole specific and
c         also executed the electronic structure programs
c       - re-written by J. Neugebauer, Uni Erlangen, 2003:
c         program execution is moved to program-specific routines
c       - extended by C. Herrmann, Uni Bonn, 02/2005 (intonly tool),
c         using code parts from ANF, version 1.1.0, written by 
c         J. Neugebauer and B.A. Hess, 2001
c       - modified by C. Herrmann, Uni Jena 04/05 (ADF-interface)
c       - modified by C. Herrmann, Uni Jena 05/05 (Gaussian interface)
c       - modified by S. Luber, ETH Zurich, June 2006 (Molpro interface)
c       - cleaned and modified by J. Neugebauer, ETHZ, 01/2007
c         (adapted for serial version)
c     ================================================================

      include 'defs.h'
      include 'comm.h'
      include 'param.h'  
      include 'objects.h'

      integer ibuf,info,nn,iat,idum,ixyz,irc,idir,l,ir1,mxat
      integer idmy,j,mbuf,diraus,ldal,iclean,icleanl,mcharge,lo
      integer ilab(natoms)
      integer*4 itst,jstepa,lsco

      real*8 co(3,natoms)
      character*255 b1
      character*10 prtype,calctype
      character*5 nic

      integer intonly,nummodes,im,imode,ipre,i,disptype,k
      real*8 nmode(maxmode,maxatoms*3),freq(maxmode)
      real*8 normmodes(maxatoms*3)
      logical found,snffast,valdtype
      external valdtype

C#    integer w,st(MPI_STATUS_SIZE)
C#    data w /MPI_COMM_WORLD/
 
      ir1=3
      lo=2

c     ------------------
c     receive step range
c     ------------------
C#    CALL MPI_RECV(itst,1,MPI_INTEGER,tid0,410,w,st,info)
C#    CALL mpierr('MPI_RECV itst','rslave',info)
C#    CALL MPI_RECV(jstepa,1,MPI_INTEGER,tid0,411,w,st,info)
C#    CALL mpierr('MPI_RECV jstepa','rslave',info)
C~    jstepa=isstep
C//SKIPC~
C//SKIPC#
      CALL PVMFRECV(tid0,0,ibuf)
      CALL PVMFUNPACK(INTEGER4,itst  ,1,1,info)
      CALL PVMFUNPACK(INTEGER4,jstepa,1,1,info)
C//END
C//END

      write (iosl,*) '>>> slave calc on tid ',mytid
      write (iosl,*) 'TID0=',tid0,' STEP ',jstepa

C----------------------
C read snf_control file
C----------------------
      call slen(tempdir(1:lt)//'/snf_control',lsco)
      call rscontrol(tempdir(1:lt)//'/snf_control',progno,calcno,ipol,
     *  omeg,snffast,iclean,icleanl,naus,mxomeg,cstep,lastqmatom,
     *  irescp,restadf,para,lsco)

C-----------------------------
C Turbomole: read control file
C-----------------------------
      if(progno.eq.1) then 
        call rcontrol(tempdir(1:lt)//'/control',rpast,smult,isuhf,
     *   natoms,idbg)

C----------------------
C Turbomole: read coord
C----------------------
        call rcoord(tempdir(1:lt)//'/coord.ori',nn,co,ilab)
        if(nn.ne.natoms) then
           write(iosl,*) 'natoms different on coord and control'
C//SKIPC~2
C//SKIPC#1
           call pvmfexit(info)
           stop
        end if

C-------------------------------
C DALTON: read molecule.mol file
C-------------------------------
      else if(progno.eq.2) then 
        open(ir1,file=tempdir(1:lt)//'/molecule.mol.ori')
        mxat = natoms
        call rwdalmol(natoms,ilab,co,ir1,mxat,mcharge,'read',lo)
        close(ir1)
C-------------------------------------------
C Gaussian: read coords from gaussian.com file
C-------------------------------------------
      else if(progno.eq.3) then
        call rgausscoord(tempdir(1:lt)//'/gaussian.com',natoms,co,
     *  ilab)
C-------------------------------
C ADF: read coords from adf.in file
C-------------------------------
      else if(progno.eq.4) then 
          write(iosl,*) 'rslave: NATOMS = ', natoms
        call radfcoord(tempdir(1:lt)//'/adf.in',natoms,co,ilab,iosl)
C----------------------------------------
C MOLPRO: read coords from molpro.in file
C----------------------------------------        
      else if(progno.eq.5) then
        call rmolpcoord(tempdir(1:lt)//'/molpro.in',natoms,co,ilab,iosl)
*----------------------
* other program modes ?
*----------------------
      else  
        write(idbg,*) 'program mode currently not available'
        call flush(idbg)
        call errex('program mode not available')
      end if

*---------------------------------------
* determine atom, xyz and dir for step i
*---------------------------------------
      if(nummodes.eq.0) then
        iat=jstepa/(3*naus)
        if(mod(jstepa,(3*naus)).ne.0.or.iat.eq.0) iat=iat+1
        idum=(jstepa-(3*naus)*(iat-1))
        ixyz=idum/naus
        if(mod(idum,naus).ne.0) ixyz=ixyz+1
        diraus=mod(idum,naus)
        if(diraus.eq.0) diraus=naus
        idir=naus/2-diraus+1
          if(idir.le.0) idir=idir-1
  
        write(iosl,*) 'step       =        ',jstepa
        write(iosl,*) 'atom       =        ',iat
        write(iosl,*) 'coordinate =        ',ixyz
        write(iosl,*) 'idir       =        ',idir

*-----------------------------------------------------
* intensities only mode: 
* determine mode no., and prefactor for displacement
* along this mode
*-----------------------------------------------------
      else
* mode number:
        im=imode(jstepa,naus)

* prefactor for displacement:
        idir=ipre(jstepa,naus)

        write(iosl,*) 'step       =        ',jstepa
        write(iosl,*) 'mode no.   =        ',im
        write(iosl,*) 'prefactor  =        ',idir
  
        i=im
*---------------------------------------------------------
* normalize normal coordinates (in principle, there should
* be normalized normal coordinates in "snf_control")
*---------------------------------------------------------
CCH        call normalize(nmode,3*natoms)
      end if

      if(nummodes.eq.0) then
*---------------------------------------------
* change coordinate - displacement of one atom
*---------------------------------------------
        write(iosl,*) 'COORD = ',co
        call flush(iosl)
        co(ixyz,iat)=co(ixyz,iat)+idir*cstep
      else
*-----------------------------------------------------
* change coordinate - displacement along normal modes 
* (intensities only program mode)
*-----------------------------------------------------
        call rkeyint(tempdir(1:lt)//'/snf_control','$disptype',disptype)
        write(iosl,*) 'disptype: ',disptype
        if(.not.valdtype(disptype)) call errex('disptype not valid')
        if(disptype.eq.1) then
          call displacecoord(nmode,co,ilab,i,idir)
        else if(disptype.eq.2) then
          call memmgr(memgdw,'vibf    ',s8,pvib,3*natoms)
          call getfrtm(tempdir(1:lt)//'/control',natoms,s8(pvib))
          call displacecoord2(normmodes,natoms,
     *         co,ilab,i,cstep,idir,s8(pvib+i-1))
          call memmgr(memfdw,'vibf    ',s8,pvib,3*natoms)
        end if
      end if

c     --------------------------------------------
c     get info about the calculation
c     --------------------------------------------
      call prog_type(progno,prtype)
      call calc_type(progno,calcno,calctype)
      write (iosl,*) 'progno : ',progno
      write (iosl,*) 'calcno : ',calcno
      write (iosl,'(2a)') ' program package for single points is: ',
     *  prtype
      write (iosl,'(2a)') ' type of calculation : ',calctype
      nic='nice '
      if(snffast) then
        write (iosl,*) '** found $snffast flag: omitting command nice'
        nic = '     '
      end if
      call flush(iosl)

c     --------------------------------------------
c     write new coordinate file 
c     --------------------------------------------
      if(progno.eq.1) then
        call wcoord(tempdir(1:lt)//'/coord',natoms,co,ilab)
      else if(progno.eq.2) then
        open(ir1,file=tempdir(1:lt)//'/molecule.mol.ori')
        open(lo,file=tempdir(1:lt)//'/molecule.mol')
        mxat = natoms
        call rwdalmol(natoms,ilab,co,ir1,mxat,mcharge,'writ',lo)
        close(lo)
        close(ir1)
      else if(progno.eq.3) then
        write(iosl,*) 'rslave: calling wgauss ... tempdir = '//
     *    tempdir(1:lt)
        call flush(iosl)
        call wgauss(tempdir(1:lt),natoms,ilab,co,iosl)
      else if(progno.eq.4) then
        write(iosl,*) 'rslave: calling wadf ... tempdir = '//
     *    tempdir(1:lt)
        call flush(iosl)
        call wadf(tempdir(1:lt),natoms,ilab,co,iosl) 
      else if(progno.eq.5) then
        write(iosl,*) 'rslave: calling wmolp ... tempdir = '//
     *    tempdir(1:lt)
        call flush(iosl)
        call wmolp(tempdir(1:lt),natoms,ilab,co)   
      end if

      return
      end 
************************************************************************
      INTEGER FUNCTION xprog(p,f)
      INCLUDE 'defs.h'
      INCLUDE 'param.h'
      CHARACTER*(*) p,f
      CHARACTER*255 cmdout
      INTEGER info,tmpxeq
      CHARACTER*24 date
      CALL fdate(date)
      CALL concat(p,' >',cmdout)
      CALL appba(cmdout,f)
C@    WRITE (iosl,*) date,cmdout
      info=tmpxeq(cmdout)
C@    WRITE (iosl,*) date,' RC=',info
      IF (info.NE.0) THEN
       WRITE (iosl,*) p,' failed; process shut down. mytid=',mytid
       CALL slshutdown(99,tid0,mytid)
      ENDIF
      xprog=info
      RETURN
      END
************************************************************************

CMR170202
      INTEGER FUNCTION xprogb(p,f)
      INCLUDE 'defs.h'
      INCLUDE 'param.h'
      CHARACTER*(*) p,f
      CHARACTER*255 cmdout
      INTEGER info,tmpxeq
      CHARACTER*24 date
      CALL fdate(date)
      CALL concat(p,' >',cmdout)
      CALL appba(cmdout,f)
C@    WRITE (iosl,*) date,cmdout
      info=tmpxeq(cmdout)
C@    WRITE (iosl,*) date,' RC=',info
      IF (info.NE.0) THEN
        WRITE (iosl,*) 'ERROR: ',p,' failed '
      ENDIF
      xprogb=info
      RETURN
      END
CMR170202
      
      subroutine actest(progn,found)
      include 'defs.h'
      include 'param.h'
      character*(*) progn
C      character*7  actual
      integer info, tmpxeq,lprog
      logical found
C      parameter(actual='$actual')
 
      found=.false.
      call llen(progn,lprog)
C ... this caused problem on a SGI machine:
C      call findstr(tempdir(1:lt)//'/control',actual(1:7),found)
      call findstr(tempdir(1:lt)//'/control','$actual',found)
      if(found) then
        write(iosl,*) '** actest: found $actual in control'
        write(iosl,*) '**         program in trouble: ',progn(1:lprog)
        call flush(iosl)
        if(lb.eq.0) then
          info=tmpxeq('actual -r')
        else
          info=tmpxeq(progdir(1:lb)//'../scripts/actual -r')
        end if
        if(info.ne.0) then
          write(iosl,*) '** actest: could not execute actual -r'
          call flush(iosl)
          call slshutdown(99,tid0,mytid)
        end if
      end if
      end
